package sample.dao;

import sample.pojo.*;
import javax.naming.*;
import javax.sql.*;
import java.sql.*;

public class CounterDAOImpl implements CounterDAO {
  private DataSource datasource;
  private static final String DATASOURCE_NAME = "java:/DefaultDS";

  public CounterDAOImpl() throws DAOException {
    try {
      InitialContext initialContext = new InitialContext();
      datasource = (DataSource) initialContext.lookup(DATASOURCE_NAME);
    } catch (NamingException e) {
      e.printStackTrace();
      throw new DAOException("DAO error");
    }
  }

  public String create(Counter counter) throws DAOException {
    Connection con = getConnection();
    PreparedStatement stmt = null;
    try {
      stmt = con.prepareStatement("insert into counter (name, value) values (?, ?)");
      stmt.setString(1, counter.getName());
      stmt.setLong(2, counter.getValue());
      stmt.executeUpdate();
    } catch (SQLException e) {
      e.printStackTrace();
      throw new DAOException("create error");
    } finally {
      closeConnection(null, stmt, con);
    }
    return counter.getName();
  }

  public Counter find(String name) throws DAOException {
    Connection con = getConnection();
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try {
      stmt = con.prepareStatement("select name, value from counter where name = ?");
      stmt.setString(1, name);
      rs = stmt.executeQuery();
      if (rs.next()) {
        long value = rs.getLong(2);
        Counter counter = new CounterImpl(name, value);
        return counter;
      } else {
        throw new CounterNotFoundException("counter notfound");
      }
    } catch (SQLException e) {
      e.printStackTrace();
      throw new DAOException("find error");
    } finally {
      closeConnection(rs, stmt, con);
    }
  }

  public void update(Counter counter) throws DAOException {
    Connection con = getConnection();
    PreparedStatement stmt = null;
    try {
      stmt = con.prepareStatement("update counter set name = ?, value = ? where name = ?");
      stmt.setString(1, counter.getName());
      stmt.setLong(2, counter.getValue());
      stmt.setString(3, counter.getName());
      stmt.executeUpdate();
    } catch (SQLException e) {
      e.printStackTrace();
      throw new DAOException("update error");
    } finally {
      closeConnection(null, stmt, con);
    }
  }

  public void delete(String name) throws DAOException {
    Connection con = getConnection();
    PreparedStatement stmt = null;
    try {
      stmt = con.prepareStatement("delete from counter where name = ?");
      stmt.setString(1, name);
      stmt.executeUpdate();
    } catch (SQLException e) {
      e.printStackTrace();
      throw new DAOException("delete error");
    } finally {
      closeConnection(null, stmt, con);
    }
  }

  private Connection getConnection() throws DAOException {
    try {
      return datasource.getConnection();
    } catch (SQLException e) {
      e.printStackTrace();
      throw new DAOException("getConnection error");
    }
  }

  private void closeConnection(ResultSet rs, Statement stmt, Connection con) {
    try {
      if (rs != null) rs.close();
      if (stmt != null) stmt.close();
      if (con != null) con.close();
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }
}
